/**
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2013-2014 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Oracle. Portions Copyright 2013-2014 Oracle. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.html.ko4j;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;
import net.java.html.BrwsrCtx;
import net.java.html.js.JavaScriptBody;
import net.java.html.json.Models;

final class ComponentLoader {
    @JavaScriptBody(args = {}, wait4js = false, javacall = true, body =
        "var myLoader = {\n" +
        "};\n" +
        "myLoader.getConfig = function(name, callback) { \n" +
        "  function myClbk(config) {\n" +
        "    console.log('myClbk: ' + config.ko4j + ' ' + config.properties);\n" +
        "    callback(config);\n" +
        "  }\n" +
        "  @org.netbeans.html.ko4j.ComponentLoader::loadConfig(Ljava/lang/String;Ljava/lang/Object;)(name, myClbk);\n" +
        "}\n" +
        "myLoader.loadComponent = function(name, componentConfig, callback) {\n" +
        "  console.log('loadComponent: ' + componentConfig.ko4j + ' ' + componentConfig.properties);\n" +
        " callback(null);\n" +
        "}\n" +
        "myLoader.loadTemplate = function(name, templateConfig, callback) {\n" +
        "  console.log('loadTemplate: ' + templateConfig.ko4j + ' ' + templateConfig.properties);\n" +
        "  if (templateConfig !== null) {\n" +
        "    @org.netbeans.html.ko4j.ComponentLoader::loadTemplate(Ljava/lang/String;Ljava/lang/String;Ljava/lang/Object;)(name, templateConfig, callback);\n" +
        "  } else {\n" +
        "    callback(null);\n" +
        "  }\n" +
        "}\n" +
        "myLoader.loadViewModel = function(name, config, callback) { \n" +
        "  console.log('loadViewModel ' + name + ' temp: ' + config);\n" +
        "  var f = function(params, info) {\n" +
        "    if (!params) params = null;\n" +
        "    return @org.netbeans.html.ko4j.ComponentLoader::loadViewModel(Ljava/lang/String;Ljava/util/Properties;Ljava/lang/Object;Ljava/lang/Object;)(name, config, params, info);\n" +
        "  };\n" +
        "  callback(f);\n" +
        "}\n" +
        "ko.components.loaders.unshift(myLoader);\n"
    )
    public static native void initialize();
    
    static void loadConfig(String name, Object callback) throws IOException {
        System.err.println("config for " + name);
        Enumeration<URL> en = ComponentLoader.class.getClassLoader().getResources("META-INF/html+java/components/" + name);
        while (en.hasMoreElements()) {
            URL u = en.nextElement();
            InputStream is = u.openStream();
            Properties p = new Properties();
            p.load(is);
            is.close();
            callbackConfig(callback, p, p.getProperty("template"));
            return;
        }
        callbackConfig(callback, null, null);
    }
    static void loadTemplate(String name, String template, Object callback) throws IOException {
        InputStream is = ComponentLoader.class.getClassLoader().getResourceAsStream(template);
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        for (;;) {
            int ch = is.read();
            if (ch == -1) {
                break;
            }
            os.write(ch);
        }
        is.close();
        String text = os.toString("UTF-8");
        callbackTemplate(callback, text);
    }
    
    @JavaScriptBody(args = { "callback", "param", "template" }, body =
        "callback(param ? {\n"
      + "  'ko4j': true,\n"
      + "  'viewModel': param,\n"
      + "  'template': template\n"
      + " } : null);", wait4js = false)
    private static native void callbackConfig(Object callback, Properties param, String template);

    @JavaScriptBody(args = { "callback", "param" }, body = "callback(ko.utils.parseHtmlFragment(param));", wait4js = false)
    private static native void callbackTemplate(Object callback, String param);
    
    static Object loadViewModel(String name, Properties props, Object params, Object config) throws IOException {
        Object res = null;
        try {
            String paramName = props.getProperty("paramName");
            Class<?> paramType = Class.forName(paramName);
            String className = props.getProperty("className");
            Class<?> classType = Class.forName(className);
            Method m = classType.getMethod(props.getProperty("methodName"), paramType);

            BrwsrCtx ctx = BrwsrCtx.findDefault(ComponentLoader.class);
            Object data;
            if (params == null) {
                data = paramType.newInstance();
            } else {
                data = Models.fromRaw(ctx, paramType, params);
            }
            m.setAccessible(true);
            res = m.invoke(null, data);
            props.put(res, res);
        } catch (Exception ex) {
            throw new IOException(ex);
        }
        return Models.toRaw(res);
    }
}
